@@ -13,6 +13,7 @@ import pandas as pd
 import numpy as np
 import hicstraw
 import cooler
+from mustache import kth_diag_indices, parseBP, read_bias, read_pd, read_hic_file, read_cooler, read_cooler, get_diags, normalize_sparse, get_sep
 
 from scipy.stats import expon, norm
 from scipy.ndimage import gaussian_filter
@@ -22,34 +23,9 @@ import scipy.ndimage.measurements as scipy_measurements
 from scipy import sparse
 
 from statsmodels.stats.multitest import multipletests
-
-
 from multiprocessing import Process, Manager
 
 
-def parseBP(s):
-    """
-    :param s: string
-    :return: string converted to number, taking account for kb or mb
-    """
-    if not s:
-        return False
-    if s.isnumeric():
-        return int(s)
-    s = s.lower()
-    if "kb" in s:
-        n = s.split("kb")[0]
-        if not n.isnumeric():
-            return False
-        return int(n) * 1000
-    elif "mb" in s:
-        n = s.split("mb")[0]
-        if not n.isnumeric():
-            return False
-        return int(n) * 1000000
-    return False
-
-
 def parse_args(args):
     parser = argparse.ArgumentParser(description="Check the help flag")
 
@@ -203,24 +179,6 @@ def parse_args(args):
                         required=False)
     return parser.parse_args()
 
-
-def kth_diag_indices(a, k):
-    rows, cols = np.diag_indices_from(a)
-    if k < 0:
-        return rows[-k:], cols[:k]
-    elif k > 0:
-        return rows[:-k], cols[k:]
-    else:
-        return rows, cols
-
-
-def is_chr(s, c):
-    #if 'X' == c or 'chrX':
-    #    return 'X' in c
-    #if 'Y' == c:
-    #    return 'Y' in c
-    return str(c).replace('chr','') == str(s).replace('chr','')#re.findall("[1-9][0-9]*", str(s))
-
 def readcstr(f):
     #*    Title: hic2cool
     #*    Author: Carl Vitzthum
@@ -291,473 +249,6 @@ def read_header(req):
     return chrs, resolutions, masterindex, genome, metadata
 
 
-def get_sep(f):
-    """
-    :param f: file path
-    :return: Guesses the value separator in the file.
-    """
-    with open(f) as file:
-        for line in file:
-            if "\t" in line:
-                return '\t'
-            if " " in line.strip():
-                return ' '
-            if "," in line:
-                return ','
-            if len(line.split(' '))==1:
-                return ' '
-            break
-    raise FileNotFoundError
-
-
-def read_bias(f, chromosome, res):
-    """
-    :param f: Path to the bias file
-    :return: Dictionary where keys are the bin coordinates and values are the bias values.
-    """
-    d = defaultdict(lambda: 1.0)
-    if f:
-        sep = get_sep(f)
-        with open(f) as file:
-            for pos,line in enumerate(file):
-                line = line.strip().split(sep)
-                if len(line)==3:
-                    if is_chr(line[0], chromosome):
-                        val = float(line[2])
-                        if not np.isnan(val):
-                            if val<0.2:
-                                d[(float(line[1]) // res)] = np.Inf
-                            else:
-                                d[(float(line[1]) // res)] = val
-                        else:
-                            d[(float(line[1]) // res)] = np.Inf
-							
-                elif len(line)==1:
-                    val = float(line[0])
-                    if not np.isnan(val):
-                        if val<0.2:
-                            d[pos] = np.Inf
-                        else:
-                            d[pos] = val
-                    else:
-                       d[pos] = np.Inf
-				
-				
-        return d
-    return False
-
-
-def read_pd(f, distance_in_bp, bias, chromosome, res):
-    sep = get_sep(f)
-    df = pd.read_csv(f, sep=sep, header=None)
-    df.dropna(inplace=True)
-    if df.shape[1]==5:       
-        df = df[np.vectorize(is_chr)(df[0], chromosome)]
-        if df.shape[0] == 0:
-            print('Could\'t read any interaction for this chromosome!')
-            return 
-        df = df[np.vectorize(is_chr)(df[2], chromosome)]
-        df = df.loc[np.abs(df[1]-df[3]) <= ((distance_in_bp/res+1) * res), :]
-        df[1] //= res
-        df[3] //= res
-        bias = read_bias(bias, chromosome, res)
-        if bias:
-            factors = np.vectorize(bias.get)(df[1], 1)
-            df[4] = np.divide(df[4], factors)
-            factors = np.vectorize(bias.get)(df[3], 1)
-            df[4] = np.divide(df[4], factors)
-
-        df = df.loc[df[4] > 0, :]
-
-        x = np.min(df.loc[:, [1, 3]], axis=1)
-        y = np.max(df.loc[:, [1, 3]], axis=1)
-        val = np.array(df[4])
-    
-    elif df.shape[1]==3:
-        df = df.loc[np.abs(df[1]-df[0]) <= ((distance_in_bp/res+1) * res), :]
-        df[0] //= res
-        df[1] //= res
-        bias = read_bias(bias, chromosome, res)
-        if bias:
-            factors = np.vectorize(bias.get)(df[0], 1)
-            df[2] = np.divide(df[2], factors)
-            factors = np.vectorize(bias.get)(df[1], 1)
-            df[2] = np.divide(df[2], factors)
-
-        df = df.loc[df[2] > 0, :]
-
-        x = np.min(df.loc[:, [0, 1]], axis=1)
-        y = np.max(df.loc[:, [0, 1]], axis=1)
-        val = np.array(df[2])  
-    
-    return x,y,val
-
-def read_hic_file(f, norm_method, CHRM_SIZE,  distance_in_bp, chr1, chr2, res):
-    """
-    :param f: .hic file path
-    :param chr: Which chromosome to read the file for
-    :param res: Resolution to extract information from
-    :return: Numpy matrix of contact counts
-    """
-    if not CHRM_SIZE:
-        hic = hicstraw.HiCFile(f)
-        chromosomes = hic.getChromosomes()
-        chrSize_in_bp = {}
-        for i in range(1, len(chromosomes)):
-            chrSize_in_bp["chr" + str(chromosomes[i].name).replace("chr", '')] = chromosomes[i].length
-        CHRM_SIZE = chrSize_in_bp["chr" + chr1.replace("chr", '')]
-    
-    CHUNK_SIZE = max(2*distance_in_bp/res, 2000)
-    start = 0
-    end = min(CHRM_SIZE, CHUNK_SIZE*res) #CHUNK_SIZE*res
-    result = []
-    val = []
-
-    while start < CHRM_SIZE:
-        print(int(start),int(end))
-        if not norm_method:
-            temp = hicstraw.straw("observed","KR", f, str(chr1)+":"+str(int(start))+":"+str(int(end)),  str(chr2)+":"+str(int(start))+":"+str(int(end)), "BP", res)
-        else:
-            temp = hicstraw.straw("observed",str(norm_method), f, str(chr1)+":"+str(int(start))+":"+str(int(end)),  str(chr2)+":"+str(int(start))+":"+str(int(end)), "BP", res)
-        if len(temp)==0:
-            start = min( start + CHUNK_SIZE*res -  distance_in_bp, CHRM_SIZE)
-            if end==CHRM_SIZE-1:
-                break
-            else:
-                end = min(end + CHUNK_SIZE*res - distance_in_bp, CHRM_SIZE-1)
-            continue
- 
-        if result == []:                                         
-            result+= [[int(record.binX), int(record.binY), record.counts] for record in temp]
-            prev_block = set([(record.binX, record.binY, record.counts) for record in temp])
-        else:
-            cur_block = set([(int(record.binX), int(record.binY), record.counts) for record in temp])
-            
-            to_add_list = list(cur_block - prev_block)
-            del prev_block
-            result[0]+=  [x[0] for x in  to_add_list]
-            result[1]+=  [x[1] for x in  to_add_list]
-            result[2]+=  [x[2] for x in  to_add_list]
-            prev_block = cur_block
-            del cur_block
-        start = min( start + CHUNK_SIZE*res -  distance_in_bp, CHRM_SIZE)
-        if end==CHRM_SIZE-1:
-            break
-        else:
-            end = min(end + CHUNK_SIZE*res - distance_in_bp, CHRM_SIZE-1)
-    
-    x = np.array(result[0]) // res
-    y = np.array(result[1]) // res
-    val = np.array(result[2])
-    nan_indx = np.logical_or.reduce((np.isnan(result[0]),np.isnan(result[1]),np.isnan(result[2])))
-    x = x[~nan_indx]
-    y = y[~nan_indx]
-    val = val[~nan_indx]
-    x = x.astype(int)
-    y = y.astype(int)
- 
-    if len(val)==0:
-        print(f'There is no contact in chrmosome {chr1} to work on.')
-        return [], [], []
-    else:
-        val[np.isnan(val)] = 0
-
-    
-
-    if(chr1==chr2):
-        dist_f = np.logical_and(np.abs(x-y) <= distance_in_bp/res, val > 0)
-        x = x[dist_f]
-        y = y[dist_f]
-        val = val[dist_f]
-
-    if len(val>0):
-        return x, y, val
-    else:
-        print(f'There is no contact in chrmosome {chr1} to work on.')
-        return [], [], []
-
-def read_cooler(f, distance_in_bp, chr1, chr2, cooler_balance):
-    """
-    :param f: .cool file path
-    :param chr: Which chromosome to read the file for
-    :return: Numpy matrix of contact counts
-    """
-    clr = cooler.Cooler(f)
-    res = clr.binsize
-    print(f'Your cooler data resolution is {res}')
-    if chr1 not in clr.chromnames or chr2 not in clr.chromnames:
-        raise NameError('wrong chromosome name!')
-    CHRM_SIZE = clr.chromsizes[chr1]
-    CHUNK_SIZE = max(2*distance_in_bp/res, 2000)
-    start = 0
-    end = min(CHUNK_SIZE*res, CHRM_SIZE) #CHUNK_SIZE*res
-    result = []
-    val = []
-    ###########################
-    if chr1 == chr2:
-        #try:
-            #normVec = clr.bins()['weight'].fetch(chr1)
-            #result = clr.matrix(balance=True,sparse=True).fetch(chr1)#as_pixels=True, join=True
-            while start < CHRM_SIZE:
-                print(int(start),int(end))
-                if not cooler_balance:
-                    temp = clr.matrix(balance=True,sparse=True).fetch( (chr1, int(start), int(end)))
-                else:
-                    temp = clr.matrix(balance=cooler_balance,sparse=True).fetch( (chr1, int(start), int(end)))
-                temp = sparse.triu(temp)
-                np.nan_to_num(temp, copy=False, nan=0, posinf=0, neginf=0)
-                start_in_px = int(start/res)
-                if len(temp.row)==0:
-                    start = min( start + CHUNK_SIZE*res -  distance_in_bp, CHRM_SIZE)
-                    if end==CHRM_SIZE-1:
-                        break
-                    else:
-                        end = min(end + CHUNK_SIZE*res - distance_in_bp, CHRM_SIZE-1)
-                    continue
-
-                if result == []:
-                    result+= [list(start_in_px+temp.row),list(start_in_px+temp.col),list(temp.data)]
-                    prev_block = set([(x,y,v) for x,y,v in zip(start_in_px+temp.row,start_in_px+temp.col,temp.data)])                    
-                else:
-                    cur_block = set([(x,y,v) for x,y,v in zip(start_in_px+temp.row,start_in_px+temp.col,temp.data)])
-                    to_add_list = list(cur_block - prev_block)
-                    del prev_block
-                    result[0]+=  [x[0] for x in  to_add_list]
-                    result[1]+=  [x[1] for x in  to_add_list]
-                    result[2]+=  [x[2] for x in  to_add_list]
-                    prev_block = cur_block
-                    del cur_block
-
-                start = min( start + CHUNK_SIZE*res -  distance_in_bp, CHRM_SIZE)
-                if end==CHRM_SIZE-1:
-                    break
-                else:
-                    end = min(end + CHUNK_SIZE*res - distance_in_bp, CHRM_SIZE-1) 
-        #except:
-            #raise NameError('Reading from the file failed!')
-            if len(result)==0:
-                print(f'There is no contact in chrmosome {chr1} to work on.')
-                return [],[],[],res
-
-            x = np.array(result[0])
-            y = np.array(result[1])
-            val = np.array(result[2])
-    else:
-       
-        result = clr.matrix(balance=True,sparse=True).fetch(chr1, chr2)
-        result = sparse.triu(result)
-        np.nan_to_num(result, copy=False, nan=0, posinf=0, neginf=0)
-        x = result.row
-        y = result.col
-        val = result.data
-    
-    ##########################
-    if len(val)==0:
-        print(f'There is no contact in chrmosome {chr1} to work on.')
-        return [],[],[],res 
-    else:
-        val[np.isnan(val)] = 0
- 
-    if(chr1==chr2):
-        dist_f = np.logical_and(np.abs(x-y) <= distance_in_bp/res, val > 0)
-        x = x[dist_f]
-        y = y[dist_f]
-        val = val[dist_f]
-    #return np.array(x),np.array(y),np.array(val), res, normVec
-    if len(val>0):
-        return np.array(x),np.array(y),np.array(val), res
-    else:
-        print(f'There is no contact in chrmosome {chr1} to work on.')
-        return [], [], [], res
-	
-def read_mcooler(f, distance_in_bp, chr1, chr2, res, cooler_balance):
-    """
-    :param f: .cool file path
-    :param chr: Which chromosome to read the file for
-    :param res: Resolution to extract information from
-    :return: Numpy matrix of contact counts
-    """
-    uri = '%s::/resolutions/%s' % (f, res)
-    #uri = '%s::/7' % (f)
-    clr = cooler.Cooler(uri)
-    #print(clr.bins()[:100])
-    if chr1 not in clr.chromnames or chr2 not in clr.chromnames:
-        raise NameError('wrong chromosome name!')
-    CHRM_SIZE = clr.chromsizes[chr1]    
-    CHUNK_SIZE = max(2*distance_in_bp/res, 2000)
-    start = 0
-    end = min(CHRM_SIZE, CHUNK_SIZE*res) #CHUNK_SIZE*res
-    result = []
-    val = []
-	
-    if chr1 == chr2:
-        try:
-            #result = clr.matrix(balance=True,sparse=True).fetch(chr1)#as_pixels=True, join=True
-            while start < CHRM_SIZE:
-                print(int(start),int(end))               
-                if not cooler_balance: 
-                    temp = clr.matrix(balance=True,sparse=True).fetch( (chr1, int(start), int(end)))
-                else:
-                    temp = clr.matrix(balance=cooler_balance,sparse=True).fetch( (chr1, int(start), int(end)))
-                temp = sparse.triu(temp)
-                np.nan_to_num(temp, copy=False, nan=0, posinf=0, neginf=0)
-                start_in_px = int(start/res)
-                if len(temp.row)==0:
-                    start = min( start + CHUNK_SIZE*res -  distance_in_bp, CHRM_SIZE)
-                    if end==CHRM_SIZE-1:
-                        break
-                    else:
-                        end = min(end + CHUNK_SIZE*res - distance_in_bp, CHRM_SIZE-1)
-
-                    continue
-           
-                if result == []:
-                    result+= [list(start_in_px+temp.row),list(start_in_px+temp.col),list(temp.data)] 
-                    prev_block = set([(x,y,v) for x,y,v in zip(start_in_px+temp.row,start_in_px+temp.col,temp.data)])
-                    #print('result==[]')
-                else:
-                    cur_block = set([(x,y,v) for x,y,v in zip(start_in_px+temp.row,start_in_px+temp.col,temp.data)])
-                    to_add_list = list(cur_block - prev_block)
-                    del prev_block
-                    result[0]+=  [x[0] for x in  to_add_list]
-                    result[1]+=  [x[1] for x in  to_add_list]
-                    result[2]+=  [x[2] for x in  to_add_list]
-                    prev_block = cur_block
-                    del cur_block
-
-                start = min( start + CHUNK_SIZE*res -  distance_in_bp, CHRM_SIZE)
-                if end==CHRM_SIZE-1:
-                    break
-                else:
-                    end = min(end + CHUNK_SIZE*res - distance_in_bp, CHRM_SIZE-1)
-        except:
-            raise NameError('Reading from the file failed!')
-        
-        if len(result)==0:
-            print(f'There is no contact in chrmosome {chr1} to work on.')
-            return [],[],[]
-
-        x = np.array(result[0])
-        y = np.array(result[1])
-        val = np.array(result[2])
-    else:
-        result = clr.matrix(balance=True,sparse=True).fetch(chr1, chr2)    
-        result = sparse.triu(result)
-        np.nan_to_num(result, copy=False, nan=0, posinf=0, neginf=0)
-        x = result.row
-        y = result.col
-        val = result.data
-    
-    if len(val)==0:
-        print(f'There is no contact in chrmosome {chr1} to work on.')
-        return [],[],[]
-    else:
-        val[np.isnan(val)] = 0
-
-    if(chr1==chr2):
-        dist_f = np.logical_and(np.abs(x-y) <= distance_in_bp/res, val > 0)
-        x = x[dist_f]
-        y = y[dist_f]
-        val = val[dist_f]
-
-    if len(val>0):
-        return np.array(x),np.array(y),np.array(val)
-    else:
-        print(f'There is no contact in chrmosome {chr1} to work on.')
-        return [], [], []    
-
-
-def get_diags(map):
-    """
-    :param map: Contact map, numpy matrix
-    :return: 2 Dictionaries where keys are the diagonal number and values are the mean of that diagonal in one dictionary and the std. in the other dictionary.
-    """
-    means = {}
-    stds = {}
-    for i in range(len(map)):
-        diag = map.diagonal(i)
-        diag = diag[diag != 0]
-        if len(diag) > 0:
-            mean = np.mean(diag)
-            std = np.std(diag) if np.std(diag) != 0 else 1
-            if math.isnan(mean):
-                means[i] = 0
-            else:
-                means[i] = mean
-            if math.isnan(std):
-                stds[i] = 1
-            else:
-                stds[i] = std
-        else:
-            means[i] = 0
-            stds[i] = 1
-    return means, stds
-
-
-def normalize_sparse(x, y, v, resolution, distance_in_px):
-    n = max(max(x),max(y)) + 1
-
-    #distance_in_px = min(distance_in_px, n)
-    distances = np.abs(y-x)
-    if (n-distance_in_px)*resolution  > 2000000: 
-        with warnings.catch_warnings():
-            warnings.simplefilter('ignore', category=RuntimeWarning)
-            filter_size = int(2000000 / resolution)
-            for d in range(2 + distance_in_px):
-                indices = distances == d
-                vals = np.zeros(n-d)
-                vals[x[indices]] = v[indices]+0.001
-                if vals.size == 0:
-                    continue
-                std = np.std(v[indices])
-                mean = np.mean(v[indices])
-                if math.isnan(mean):
-                    mean = 0
-                if math.isnan(std):
-                    std = 1
-
-                kernel = np.ones(filter_size)
-                counts = np.convolve(vals != 0, kernel, mode='same')
-
-                s = np.convolve(vals, kernel, mode='same')
-                s2 = np.convolve(vals ** 2, kernel, mode='same')
-                local_var = (s2 - s ** 2 / counts) / (counts - 1)
-
-                std2 = std ** 2
-                np.nan_to_num(local_var, copy=False,
-                          neginf=std2, posinf=std2, nan=std2)
-
-                local_mean = s / counts
-                local_mean[counts < 30] = mean
-                local_var[counts < 30] = std2
-			
-                np.nan_to_num(local_mean, copy=False,
-                          neginf=mean, posinf=mean, nan=mean)
-
-                local_std = np.sqrt(local_var)
-                vals[x[indices]] -= local_mean[x[indices]]
-                vals[x[indices]] /= local_std[x[indices]]
-                np.nan_to_num(vals, copy=False, nan=0, posinf=0, neginf=0)
-                vals = vals*(1 + math.log(1+mean, 30))
-            
-                v[indices] = vals[x[indices]]
-    else:
-        with warnings.catch_warnings():
-            warnings.simplefilter('ignore', category=RuntimeWarning)
-            np.nan_to_num(v, copy=False, neginf=0, posinf=0, nan=0)
-            distance_in_px = min(distance_in_px, n)
-            for d in range(distance_in_px):
-                indices = distances == d                                    
-                std = np.std(v[indices])
-                mean = np.mean(v[indices])
-                if math.isnan(mean):
-                    mean = 0
-                if math.isnan(std):
-                    std = 1
-            #print(std)
-                v[indices] = (v[indices] - mean)/std
-                np.nan_to_num(v, copy=False, nan=0, posinf=0, neginf=0) 
-			
 def inter_nrmalize_map(vals):
     m = np.mean(vals)
     s = np.std(vals)
@@ -1300,12 +791,12 @@ def main():
             if clr.chromsizes[i]>1000000:
                 chr_list.append(chrm)
     elif CHR_HIC_FLAG:
-        hic = open(f1, 'rb')
-        chrs, resolutions, masterindex, genome, metadata = read_header(hic)
-        chr_list = [chrs[i][1] for i in range(1,len(chrs))]
+        hic = hicstraw.HiCFile(f)
+        chromosomes = hic.getChromosomes()
+        chr_list = [chromosomes[i].name for i in range(1, len(chromosomes))]
         chrSize_in_bp = {}
-        for i in range(1,len(chrs)):        
-            chrSize_in_bp["chr"+str(chrs[i][1]).replace("chr",'')] = chrs[i][2] 
+        for i in range(1, len(chromosomes)):
+            chrSize_in_bp["chr" + str(chromosomes[i].name).replace("chr", '')] = chromosomes[i].length
     else:
         chr_list = args.chromosome.copy()        
 
